/*
 * Copyright © 2021-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel24-VierConnects-core.
 *
 * ipspiel24-VierConnects-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel24-VierConnects-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel24-VierConnects-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel24.VierConnects.core.domain.impl;

import java.util.Objects;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsBoard;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsDirection;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsField;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsFieldState;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsPosition;

/**
 * Implements {@link VierConnectsField}.
 */
final class VierConnectsFieldImpl implements VierConnectsField {

    /**
     * The board this field belongs to.
     */
    private final VierConnectsBoardImpl board;
    /**
     * The position at which this field is placed on the board.
     */
    private final VierConnectsPosition position;
    /**
     * The state of this field.
     */
    private VierConnectsFieldState state;

    /**
     * Creates an VierConnects field.
     *
     * @param board    The board this field belongs to.
     * @param position The position at which this field is placed on the board.
     * @param state    The state of this field.
     */
    VierConnectsFieldImpl(final VierConnectsBoardImpl board, final VierConnectsPosition position,
            final VierConnectsFieldState state) {
        this.board = Objects.requireNonNull(board, "board");
        this.position = Objects.requireNonNull(position, "position");
        this.state = state;
    }

    @Override
    public String toString() {
        return String.format("VierConnectsField[position=%s, state=%s]", this.position, this.state);
    }

    /**
     * {@inheritDoc}
     * <p>
     * Does not compare the boards the fields belong to, respectively.
     */
    @Override
    public boolean equals(final Object obj) {
        if (obj instanceof VierConnectsFieldImpl) {
            final VierConnectsFieldImpl other = (VierConnectsFieldImpl) obj;
            return this.position.equals(other.position) && this.state.equals(other.state);
        }
        return false;
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.position, this.state);
    }

    @Override
    public VierConnectsBoard getBoard() {
        return this.board;
    }

    @Override
    public VierConnectsPosition getPosition() {
        return this.position;
    }

    @Override
    public VierConnectsFieldState getState() {
        return this.state;
    }

    @Override
    public boolean hasNeighbour(final VierConnectsDirection direction) {
        return this.board.hasFieldAt(direction.step(this.position));
    }

    @Override
    public VierConnectsField getNeighbour(final VierConnectsDirection direction) {
        if (!this.hasNeighbour(direction)) {
            throw new IllegalArgumentException(String.format("No %s neighbour at %s.", direction, this.position));
        }

        return this.board.getFieldAt(direction.step(this.position));
    }

    @Override
    public void changeState(final VierConnectsFieldState newState) throws GameException {
        final VierConnectsFieldState oldState = this.state;
        final boolean wasEmpty = oldState.equals(VierConnectsFieldState.EMPTY);
        final boolean isEmpty = newState.equals(VierConnectsFieldState.EMPTY);

        if (wasEmpty && isEmpty) {
            throw new GameException(String.format("It is not allowed to do nothing on field at %s.", this.position));
        }
        if (isEmpty) {
            throw new GameException(
                    String.format("It is not allowed to remove a mark from field at %s.", this.position));
        }
        if (!wasEmpty) {
            throw new GameException(String.format("It is not allowed to change a mark on field at %s.", this.position));
        }

        this.state = newState;
    }
}
